package basicTurtlePackage;


import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.util.ArrayList;

import javax.swing.JComponent;

/**  
 * File:	Turtle.java - a class that represents a turtle.
 * 			The turtle can move and draw on the screen.
 * Date:	January 25, 2010
 * @author  Aleksandar Stefanovski
 * @version 1.0 
 * 
 * Note:	This turtle class has been modeled after the turtles
 * 			from Barbara Ericson's book: "Exploring Wonderland
 * 			Java Programming Using Alice and Media Computation" 1st edition.
 */ 
public class Turtle {

	// variables
	protected World myWorld;
	protected String myName;
	// position variables
	protected int x, y;
	// heading can be anywhere from 0 - 360 (positive or negative)
	private double heading = 0;  
	protected boolean visible;

	protected int size;
	// color
	protected Color shellColor, bodyColor, penColor;

	// Pen related variables
	protected boolean penDown;
	protected int penWidth;
	protected ArrayList<int[]> penXY;

	/**  
	 * Method:	Constructor method.
	 * Purpose: Creates a new object (instance) of this class.  
	 * 
	 * @param	none	
	 * @return 	none 
	 */  
	public Turtle(World newWorld) {
		myWorld = newWorld;
		myName = "Turtle";
		// set the size (default)
		size = 30;

		// set the turtle's starting position
		x = myWorld.getWidth()/2;
		y = myWorld.getHeight()/2;

		// set the heading, default is 0
		this.setHeading(0);

		// set the color
		this.setColor(Color.green);


		// set the penDo by default
		this.penDown();
		this.setPenWidth(2);
		this.penXY = new ArrayList<int[]>();

		this.visible = true;
		// finally add the turtle to the world
		myWorld.add(this);

		// creating a pause of 1 second to animate the creation
		this.pause(0);
	} // end constructor

	/**  
	 * Method:	paint
	 * Purpose: Gives the ability to paint on the screen.  
	 * 
	 * @param	g (Graphics)	
	 * @return 	none 
	 */
	public void paint(Graphics g) {

		// cast to 2d object
		Graphics2D g2 = (Graphics2D) g;
		// get the center of the turtle as the rotational center
		int offset = size/2;
		int oX = x - offset;
		int oY = y - offset;
		// if the turtle is visible
		if (visible)
		{
			// save the current transform
			AffineTransform oldTransform = g2.getTransform();

			// rotate the turtle and translate to the center x,y of the turtle
			g2.rotate(Math.toRadians(getHeading()),oX+size/2,oY+size/2);

			// draw head
			int headSize = size/3;
			g2.setColor(bodyColor);
			g2.fillOval(oX+size/2-headSize/2,oY-headSize,headSize,headSize);	
			g2.setColor(shellColor);
			g2.drawOval(oX+size/2-headSize/2,oY-headSize,headSize,headSize);	
			// draw front left leg
			int legSize = size/4;
			g2.setColor(bodyColor);
			g2.fillOval(oX,oY,legSize,legSize);
			g2.setColor(shellColor);
			g2.drawOval(oX,oY,legSize,legSize);
			// draw front right leg
			g2.setColor(bodyColor);
			g2.fillOval(oX + size - legSize, oY, legSize, legSize);
			g2.setColor(shellColor);
			g2.drawOval(oX + size - legSize, oY, legSize, legSize);
			// draw back left leg
			g2.setColor(bodyColor);
			g2.fillOval(oX, oY + size - legSize, legSize, legSize);
			g2.setColor(shellColor);
			g2.drawOval(oX, oY + size - legSize, legSize, legSize);
			// draw back right leg
			g2.setColor(bodyColor);
			g2.fillOval(oX + size - legSize, oY + size - legSize, legSize, legSize);
			g2.setColor(shellColor);
			g2.drawOval(oX + size - legSize, oY + size - legSize, legSize, legSize);
			// draw body
			g2.setColor(bodyColor);
			g2.setColor(shellColor);
			g2.fillOval(oX,oY, size, size);
			g2.drawOval(oX,oY, size, size);

			// reset the transformation matrix
			g2.setTransform(oldTransform);
		} // end if

		BasicStroke penStroke = new BasicStroke(penWidth); 
		g2.setStroke(penStroke);
		for(int[] eachPenPos:penXY) {

			g2.setColor(this.penColor);
			g2.drawLine(eachPenPos[0], eachPenPos[1], eachPenPos[2], eachPenPos[3]);
		} // end for loop
		penStroke = new BasicStroke(1); 
		g2.setStroke(penStroke);

	} // end paint

	/**  
	 * Method:	forward
	 * Purpose: The turtle moves forward 100 pixels (steps) by default.  
	 * 
	 * @param	none	
	 * @return 	none 
	 */
	public void forward() {
		this.forward(100);
	} // end forward

	/**  
	 * Method:	forward
	 * Purpose: The turtle moves forward a number of pixels (steps).  
	 * 
	 * @param	distance (int)	
	 * @return 	none 
	 */
	public void forward(int distance) {
		// turtle can refuse to move out of 
		// the boundaries of the world 
		int oldX = x;
		int oldY = y;

		x = oldX + (int) (distance * Math.sin(Math.toRadians(getHeading())));
		y = oldY + (int) (distance * -Math.cos(Math.toRadians(getHeading())));

		if(penDown == true) {
			int[] newPos = new int[4];
			newPos[0] = oldX;
			newPos[1] = oldY;
			newPos[2] = x;
			newPos[3] = y;

			penXY.add(newPos);
		} // end if
		// repaint the world to show the changes
		myWorld.repaint();

		// creating a pause of 1 second to animate movements
		this.pause(1);
	} // end forward

	/**  
	 * Method:	turnRight
	 * Purpose: The turtle turns right.  
	 * 
	 * @param	none	
	 * @return 	none 
	 */
	public void turnRight() {
		this.turn(90);
	} // end turnRight

	/**  
	 * Method:	turnLeft
	 * Purpose: The turtle turns left.  
	 * 
	 * @param	none	
	 * @return 	none 
	 */
	public void turnLeft() {
		this.turn(-90);
	} // end turnLeft

	/**  
	 * Method:	turnRight
	 * Purpose: The turtle turns right.  
	 * 
	 * @param	none	
	 * @return 	none 
	 */
	public void turn(int degrees) {
		this.setHeading((getHeading() + degrees) % 360);
		// repaint the world to show the changes
		myWorld.repaint();

		// creating a pause of 1 second to animate the turns
		this.pause(1);
	} // end turn

	/**  
	 * Method:	penUp
	 * Purpose: Sets the pen up for not drawing the trail 
	 * 
	 * @param	none	
	 * @return 	none 
	 */
	public void penUp() {
		penDown = false;
	} // end penUp

	/**  
	 * Method:	penDown
	 * Purpose: Sets the pen down for drawing the trail 
	 * 
	 * @param	none	
	 * @return 	none 
	 */
	public void penDown() {
		penDown = true;
	} // end penDown

	/**  
	 * Method:	setPenWidth
	 * Purpose: Changes the width of the Pen stroke 
	 * 
	 * @param	newPenWidth (int)	
	 * @return 	none 
	 */
	public void setPenWidth(int newPenWidth) {
		penWidth = newPenWidth;
	} // end setPenWidth

	/**  
	 * Method:	setColor
	 * Purpose: Changes the turtle's colors to a new color 
	 * 
	 * @param	newColor (Color)	
	 * @return 	none 
	 */
	public void setColor(Color newColor) {
		this.setBodyColor(newColor);
		this.setShellColor(newColor.darker());
		myWorld.repaint();
		// creating a pause of 1 second to animate the change of the color
		this.pause(1);
	} // end setColor

	/**  
	 * Method:	setBodyColor
	 * Purpose: Changes the turtle's body color to a new color 
	 * Side-effect: it also set's the pen color to the new color
	 * @param	newColor (Color)	
	 * @return 	none 
	 */
	public void setBodyColor(Color newColor) {
		this.bodyColor = newColor;
		this.setPenColor(this.bodyColor);
	} // end setBodyColor

	/**  
	 * Method:	setShellColor
	 * Purpose: Changes the turtle's shell color to a new color 
	 * 
	 * @param	newColor (Color)	
	 * @return 	none 
	 */
	public void setShellColor(Color newColor) {
		this.shellColor = newColor;
	} // end setShellColor
	
	/**  
	 * Method:	setPenColor
	 * Purpose: Changes the Pen color to a new color 
	 * 
	 * @param	newColor (Color)	
	 * @return 	none 
	 */
	public void setPenColor(Color newColor) {
		this.penColor = newColor;
	} // end setPenColor

	/**  
	 * Method:	moveTo
	 * Purpose: changes the turtle's x and y coordinates
	 * 			and repaints the screen to instantly display
	 * 			the turtle's new position.  
	 * 
	 * @param	seconds (double)	
	 * @return 	none 
	 */
	public void moveTo(int pX, int pY) {
		this.x = pX;
		this.y = pY;
		myWorld.repaint();
		// creating a pause of 1 second to animate the movement
		this.pause(1);
	} // end moveTo

	/**
	 * Method to return a string with information 
	 * about this turtle
	 * @return a string with information about this object
	 */
	public String toString() {
		return this.myName + " turtle at " + this.x + ", " + 
		this.y + " heading " + this.getHeading() + ".";
	} // end toString

	/**  
	 * Method:	pause
	 * Purpose: Gives the ability to simulate animation
	 * 			while displaying on the screen.  
	 * 
	 * @param	seconds (double)	
	 * @return 	none 
	 */
	protected void pause(double seconds) {
		try {
			Thread.sleep((int)(100 * seconds));
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	} // end pause
	
	/**  
	 * Method:	setSize
	 * Purpose: Changes the size of the turtle to a new size.  
	 * 
	 * @param	newSize (int)	
	 * @return 	none 
	 */
	public void setSize(int newSize) {
		this.size = newSize;
	} // end setSize

	public void setHeading(double heading) {
		this.heading = heading;
	}

	public double getHeading() {
		return heading;
	}

	public int getSize() {
		return this.size;
	}
	
}
